"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.checkPropsObjectTypeMatchesSchema = checkPropsObjectTypeMatchesSchema;
exports.toGraphQLQueryString = toGraphQLQueryString;

var _flow = require("./flow");

var _types = require("./types");

var _graphql = require("graphql");

function checkPropsObjectTypeMatchesSchema(schema, fragmentName, objectType) {
  var graphqlType = schema.getType(fragmentName);
  if (!graphqlType || !(graphqlType instanceof _graphql.GraphQLObjectType)) {
    throw new Error("There is no GraphQL object type named '" + fragmentName + "'");
  }

  var typeAnnotation = objectType.value;
  var flowTypes = typeAnnotation.type === "ObjectTypeAnnotation" ? (0, _flow.convertFlowObjectTypeAnnotation)(typeAnnotation) : {};
  var graphqlTypes = convertGraphqlObjectType(graphqlType);

  var result = compareFlowTypes(flowTypes, graphqlTypes);
  if (result.length > 0) {
    var errors = result.map(function (r) {
      return "Expected type '" + r.expected + "', actual type '" + r.actual + "' for path " + r.path.join(".");
    });
    throw new Error("Errors for fragment " + fragmentName + ":\n" + errors.join("\n"));
  }
}

function convertGraphqlObjectType(objectType) {
  var fields = objectType.getFields();
  return Object.keys(fields).reduce(function (obj, key) {
    var field = fields[key];
    obj[key] = graphqlFieldToString(field);
    return obj;
  }, {});
}

function graphqlFieldToString(field) {
  var nullable = true;
  var graphqlType = field.type;
  if (graphqlType instanceof _graphql.GraphQLNonNull) {
    nullable = false;
    graphqlType = graphqlType.ofType;
  }

  if (graphqlType instanceof _graphql.GraphQLObjectType) {
    return {
      type: "object",
      nullable: nullable,
      properties: convertGraphqlObjectType(graphqlType)
    };
  }

  var flowTypeName = graphqlType instanceof _graphql.GraphQLScalarType ? graphqlType.name : "any";
  switch (flowTypeName) {
    case "Boolean":
      return { type: "boolean", nullable: nullable };
    case "Int":
    case "Float":
      return { type: "number", nullable: nullable };
    case "ID":
    case "String":
      return { type: "string", nullable: nullable };
    default:
      return { type: "any", nullable: nullable };
  }
}

function compareFlowTypes(a, b) {
  var path = arguments.length <= 2 || arguments[2] === undefined ? [] : arguments[2];

  var createPath = function createPath(key) {
    var pathCopy = path.slice();
    pathCopy.push(key);
    return pathCopy;
  };

  var typeToString = function typeToString(type) {
    return (type.nullable ? "?" : "") + type.type;
  };

  var aKeys = Object.keys(a).sort();
  return aKeys.reduce(function (result, key) {
    // babel relay plugin handles missing fields
    if (b[key]) {
      var aType = a[key];
      var bType = b[key];

      var _actual = typeToString(aType);
      var _expected = typeToString(bType);

      if (_actual !== _expected) {
        result.push({ path: createPath(key), actual: _actual, expected: _expected });
      } else if (aType.type === "object" && bType.type === "object") {
        return result.concat(compareFlowTypes(aType.properties, bType.properties, createPath(key)));
      }
    }
    return result;
  }, []);
}

function toGraphQLQueryString(fragmentName, objectType, relayContainerFragments) {
  var fragmentKey = objectType.key.name;
  var typeAnnotation = objectType.value;
  var obj = typeAnnotation.type === "ObjectTypeAnnotation" ? (0, _flow.convertFlowObjectTypeAnnotation)(typeAnnotation) : {};

  var graphQlQueryBody = objectToGraphQLString(obj);

  var childRelayContainersForFragment = Object.keys(relayContainerFragments).reduce(function (c, name) {
    var fragmentKeys = relayContainerFragments[name];
    if (fragmentKeys.indexOf(fragmentKey) >= 0) {
      c.push(name);
    }
    return c;
  }, []);

  var graphQlQuery = "\n" + "fragment on " + fragmentName + " {\n" + graphQlQueryBody;
  var graphQlQueryEnd = "\n}\n";

  if (childRelayContainersForFragment.length > 0) {
    if (graphQlQueryBody) {
      graphQlQuery = graphQlQuery + ",\n  ";
    }

    graphQlQuery = graphQlQuery + childRelayContainersForFragment.map(function (name) {
      return "${" + name + ".getFragment('" + fragmentKey + "')}";
    }).join(",\n  ");
  }
  graphQlQuery = graphQlQuery + graphQlQueryEnd;

  return "() => Relay.QL`" + graphQlQuery + "`";
}

function objectToGraphQLString(obj) {
  var level = arguments.length <= 1 || arguments[1] === undefined ? 1 : arguments[1];

  var indentation = "  ".repeat(level);

  var strings = Object.keys(obj).reduce(function (parts, key) {
    var value = obj[key];
    var str = key;
    if (value.type === "object") {
      str = str + " {\n" + objectToGraphQLString(value.properties, level + 1) + "\n" + indentation + "}";
    }
    parts.push(indentation + str);
    return parts;
  }, []);

  return strings.join(",\n");
}